@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -42,7 +42,7 @@ USA.
 	  (begin
 	    (edges-disconnect-right! previous-edges)
 	    (edge-disconnect! next-edge)
-	    (with-values
+	    (call-with-values
 		(lambda ()
 		  (order-subproblems/application
 		   (parallel-application-node parallel)
@@ -127,7 +127,7 @@ USA.
 	  (operands
 	   (list-filter-indices (cdr subproblems) (inliner/operands inliner))))
       (set-inliner/operands! inliner operands)
-      (with-values
+      (call-with-values
 	  (lambda ()
 	    (discriminate-items operands subproblem-simple?))
 	(lambda (simple complex)
@@ -195,14 +195,14 @@ USA.
 
 (define (order-subproblems/out-of-line combination subproblems rest)
   (let ((alist (add-defaulted-subproblems! combination subproblems)))
-    (with-values
+    (call-with-values
 	(combination-ordering (combination/context combination)
 			      (car subproblems)
 			      (cdr subproblems)
 			      (combination/model combination))
       (lambda (effect-subproblems push-subproblems)
 	(set-combination/frame-size! combination (length push-subproblems))
-	(with-values
+	(call-with-values
 	    (lambda ()
 	      (order-subproblems/maybe-overwrite-block
 	       combination push-subproblems rest alist
@@ -238,12 +238,10 @@ USA.
 				(not (procedure-original-rest model))))
 		       (warn "wrong number of arguments"
 			     n-supplied
-			     (error-irritant/noise char:newline)
-			     (error-irritant/noise "in call to procedure")
+			     (error-irritant/noise "\nin call to procedure")
 			     (procedure-name model)
-			     (error-irritant/noise char:newline)
 			     (error-irritant/noise
-			      "minimum/maximum number of arguments:")
+			      "\nminimum/maximum number of arguments:")
 			     n-required
 			     n-expected))
 		   (- n-expected n-supplied))))
@@ -265,7 +263,7 @@ USA.
 		 parameters
 		 (let ((n-parameters (length parameters)))
 		   (if (> (length arguments) n-parameters)
-		       (list-head arguments n-parameters)
+		       (take arguments n-parameters)
 		       arguments)))))
 	'())))
 
@@ -297,7 +295,7 @@ USA.
 		(if (and (procedure/open? model)
 			 (stack-block/static-link? model-block))
 		    (lambda ()
-		      (with-values thunk
+		      (call-with-values thunk
 			(lambda (effect-subproblems push-subproblems)
 			  (values
 			   effect-subproblems
@@ -308,7 +306,7 @@ USA.
 	standard)))
 
 (define (optimized-combination-ordering context operator operands callee)
-  (with-values
+  (call-with-values
       (lambda ()
 	(sort-subproblems/out-of-line operands callee))
     (lambda (integrated non-integrated)
@@ -364,14 +362,14 @@ USA.
 	    subproblems))
 
 (define (sort-subproblems/out-of-line all-subproblems callee)
-  (with-values
+  (call-with-values
       (lambda ()
 	(sort-integrated (cdr (procedure-original-required callee))
 			 all-subproblems
 			 '()
 			 '()))
     (lambda (subproblems integrated non-integrated)
-      (with-values
+      (call-with-values
 	  (lambda ()
 	    (sort-integrated (procedure-original-optional callee)
 			     subproblems
