@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -82,6 +82,10 @@ USA.
   (indentation c:line-indentation)
   (text c:line-text))
 
+(define-print-method c:line?
+  (lambda (line port)
+    (c:write-line line port)))
+
 (define-guarantee c:line "C line")
 
 (define (c:line . items)
@@ -97,8 +101,15 @@ USA.
 (define (c:line-item item)
   (cond ((string? item) item)
 	((char? item) (string item))
-	((symbol? item) (symbol-name item))
-	((number? item) (number->string item))
+	((symbol? item) (symbol->string item))
+	((number? item)
+	 ;; XXX Kludgey test for negative zero, to support building
+	 ;; from versions when NUMBER->STRING failed to do that itself.
+	 (if (and (flo:flonum? item)
+		  (flo:zero? item)
+		  (flo:negative? (flo:atan2 item -1.)))
+	     "-0."
+	     (number->string item)))
 	((decoded-time? item) (decoded-time->iso8601-string item))
 	((not item) "false")
 	((eq? item #t) "true")
@@ -139,6 +150,10 @@ USA.
     c:group?
   (lines c:group-lines))
 
+(define-print-method c:group?
+  (lambda (group port)
+    (c:write-group group port)))
+
 (define-guarantee c:group "C group")
 
 (define (c:group . items)
@@ -216,13 +231,13 @@ USA.
                 (lambda (port)
                   (let ((end (string-length comment)))
                     (let loop ((start 0) (index index))
-                      (write-substring comment start index port)
+                      (write-string comment port start index)
                       (write-string "*\\/" port)
                       (let ((index (+ index 2)))
                         (cond ((substring-search-forward "*/" comment index end)
                                => (lambda (index*) (loop index index*)))
                               (else
-                               (write-substring comment index end port))))))))))
+                               (write-string comment port index end))))))))))
         (else comment)))
 
 (define (c:string . content)
@@ -361,13 +376,13 @@ USA.
 	      1
 	      0)))
       (if (fix:< i n)
-	  (and (char-set-member? c:decimal-chars (string-ref e i))
+	  (and (char-in-set? (string-ref e i) c:decimal-chars)
 	       (loop (fix:+ i 1)))
 	  #t))))
 
 (define c:decimal-chars
-  (ascii-range->char-set (char->integer #\0)
-			 (+ (char->integer #\9) 1)))
+  (ucs-range->char-set (char->integer #\0)
+		       (+ (char->integer #\9) 1)))
 
 (define (c:type type)
   (or (and (symbol? type)
@@ -380,11 +395,11 @@ USA.
   (let ((types
 	 (let ((types '(char short int long float double)))
 	   `(,@(map (lambda (t)
-		      (cons t (symbol-name t)))
+		      (cons t (symbol->string t)))
 		    types)
 	     ,@(map (lambda (t)
 		      (cons (symbol 'u t)
-			    (string-append "unsigned " (symbol-name t))))
+			    (string-append "unsigned " (symbol->string t))))
 		    types)
 	     (sobj . "SCHEME_OBJECT")))))
     `(,@types
@@ -395,12 +410,19 @@ USA.
 
 (define (c:decl type var #!optional val)
   (c:line (c:type type) " " (c:var var)
-	  (if (default-object? val) "" (string-append " = " (c:expr val)))
+	  (if (default-object? val)
+	      (string-append " LIARC_UNINITIALIZED(" (c:var var) ")")
+	      (string-append " = " (c:expr val)))
 	  ";"))
 
+(define (c:decl-unused type var #!optional val)
+  (c:line (c:type type) " " (c:var var)
+	  (if (default-object? val) "" (string-append " = " (c:expr val)))
+	  " ATTRIBUTE((unused));"))
+
 (define (c:var item)
   (cond ((string? item) item)
-	((symbol? item) (symbol-name item))
+	((symbol? item) (symbol->string item))
 	(else (error:wrong-type-argument item "C variable" 'C:VAR))))
 
 (define (c:array-decl type name dim items)
@@ -437,17 +459,17 @@ USA.
   (let ((n (string-length e)))
     (let loop ((i 0))
       (if (fix:< i n)
-	  (and (char-set-member? c:identifier-chars (string-ref e i))
+	  (and (char-in-set? (string-ref e i) c:identifier-chars)
 	       (loop (fix:+ i 1)))
 	  #t))))
 
 (define c:identifier-chars
-  (char-set-union (ascii-range->char-set (char->integer #\A)
-					 (+ (char->integer #\Z) 1))
-		  (ascii-range->char-set (char->integer #\a)
-					 (+ (char->integer #\z) 1))
-		  (ascii-range->char-set (char->integer #\0)
-					 (+ (char->integer #\9) 1))
+  (char-set-union (ucs-range->char-set (char->integer #\A)
+				       (+ (char->integer #\Z) 1))
+		  (ucs-range->char-set (char->integer #\a)
+				       (+ (char->integer #\z) 1))
+		  (ucs-range->char-set (char->integer #\0)
+				       (+ (char->integer #\9) 1))
 		  (char-set #\_)))
 
 (define (c:%parenthesized? e)
