@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -28,8 +28,9 @@ USA.
 
 (declare (usual-integrations))
 
-(let* ((scm-file (lambda (file) (string-append file ".scm")))
-       (bin-file (lambda (file) (string-append file ".bin")))
+(let* ((sf-names (lambda (file) (sf/pathname-defaulting file #f #f)))
+       (scm-file (lambda (file) (receive (scm bin spec) (sf-names file) scm)))
+       (bin-file (lambda (file) (receive (scm bin spec) (sf-names file) bin)))
        (bin-time (lambda (file) (file-modification-time (bin-file file))))
        (sf-dependent
 	(lambda (environment)
@@ -37,9 +38,10 @@ USA.
 	    (let ((reasons
 		   (let ((source-time (bin-time source)))
 		     (append
-		      (if (not (file-processed? source "scm" "bin"))
-			  (list (scm-file source))
-			  '())
+		      (receive (scm bin spec) (sf-names source)
+			(if (file-modification-time<=? scm bin)
+			    '()
+			    (list scm)))
 		      (map bin-file
 			   (if source-time
 			       (filter (lambda (dependency)
@@ -109,6 +111,7 @@ USA.
     (for-each (lambda (filename)
 		(apply sf-edwin filename includes))
 	      '("abbrev"
+                "adapters"
 		"argred"
 		"artdebug"
 		"autold"
@@ -166,7 +169,6 @@ USA.
 		"kmacro"
 		"lincom"
 		"linden"
-		"lisppaste"
 		"loadef"
 		"lspcom"
 		"malias"
