@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 Copyright (C) 2005, 2006, 2007, 2008 Taylor R. Campbell
 
 This file is part of MIT/GNU Scheme.
@@ -261,7 +261,7 @@ USA.
   (let ((cache (get-property entity 'MIME-ENTITY-BODY-STRUCTURE #f)))
     (if cache
         (let ((body-structure (weak-car cache)))
-          (if (weak-pair/car? cache)
+          (if (and body-structure (not (gc-reclaimed-object? body-structure)))
               body-structure
               (next (lambda (value) (weak-set-car! cache value)))))
         (next (lambda (value)
