@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -28,6 +28,8 @@ USA.
 ;;; package: (runtime os-primitives)
 
 (declare (usual-integrations))
+
+(add-boot-deps! '(runtime hash-table))
 
 (define (get-environment-variable name)
   (guarantee string? name 'get-environment-variable)
@@ -54,7 +56,8 @@ USA.
 (define (reset-environment-variables!)
   (hash-table-clear! %env-cache)
   (vector-for-each (lambda (s)
-		     (let ((i (string-find-next-char s #\=)))
+		     (let ((s (string-from-primitive s))
+			   (i (string-find-next-char s #\=)))
 		       (if i
 			   (hash-table-set! %env-cache
 					    (string-head s i)
