@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -29,38 +29,45 @@ USA.
 
 (declare (usual-integrations))
 
-(define repl:allow-restart-notifications? #t)
-(define repl:write-result-hash-numbers? #t)
+(add-boot-deps! '(runtime dynamic) '(runtime error-handler))
 
-(define (initialize-package!)
-  (set! *nearest-cmdl* #f)
-  (set! hook/repl-read default/repl-read)
-  (set! hook/repl-eval default/repl-eval)
-  (set! hook/repl-write default/repl-write)
-  (set! hook/set-default-environment default/set-default-environment)
-  (set! hook/error-decision #f)
-  (initialize-breakpoint-condition!)
-  unspecific)
+(define repl:allow-restart-notifications? #t)
+(define repl:write-result-hash-numbers? #f)
+
+(add-boot-init!
+ (lambda ()
+   (set! param:nearest-cmdl (make-unsettable-parameter #f))
+   (set! hook/repl-read default/repl-read)
+   (set! hook/repl-eval default/repl-eval)
+   (set! hook/repl-write default/repl-write)
+   (set! hook/set-default-environment default/set-default-environment)
+   (set! hook/error-decision #f)
+   (initialize-breakpoint-condition!)))
 
 (define (initial-top-level-repl)
   (call-with-current-continuation
    (lambda (continuation)
      (set! root-continuation continuation)
      (repl/start (make-repl #f
-			    console-i/o-port
+			    (console-i/o-port)
 			    user-initial-environment
 			    #f
-			    `((SET-DEFAULT-DIRECTORY
+			    `((set-default-directory
 			       ,top-level-repl/set-default-directory))
 			    user-initial-prompt)
-		 (cmdl-message/strings "Cold load finished")))))
+		 (cmdl-message/append
+		  (cmdl-message/active
+		   (lambda (port)
+		     (declare (ignore port))
+		     (cold-load-finished!)))
+		  (cmdl-message/strings "Cold load finished"))))))
 
 (define root-continuation)
 
 (define (top-level-repl/set-default-directory cmdl pathname)
   cmdl
   ((ucode-primitive set-working-directory-pathname! 1)
-   (->namestring pathname)))
+   (string-for-primitive (->namestring pathname))))
 
 ;;;; Command Loops
 
@@ -71,27 +78,25 @@ USA.
   (parent cmdl/parent)
   (port cmdl/port set-cmdl/port!)
   (driver cmdl/driver)
-  (state cmdl/state set-cmdl/state!)
+  (state cmdl/state)
   (operations cmdl/operations)
   (properties cmdl/properties))
 
-(define-guarantee cmdl "command loop")
-
 (define (make-cmdl parent port driver state operations)
   (if (not (or (not parent) (cmdl? parent)))
-      (error:wrong-type-argument parent "cmdl" 'MAKE-CMDL))
+      (error:wrong-type-argument parent "cmdl" 'make-cmdl))
   (if (not (or parent port))
-      (error:bad-range-argument port 'MAKE-CMDL))
+      (error:bad-range-argument port 'make-cmdl))
   (%make-cmdl (if parent (+ (cmdl/level parent) 1) 1)
 	      parent
 	      (or port (and parent (cmdl/child-port parent)))
 	      driver
 	      state
-	      (parse-operations-list operations 'MAKE-CMDL)
+	      (parse-operations-list operations 'make-cmdl)
 	      (make-1d-table)))
 
 (define (cmdl/child-port cmdl)
-  (or (let ((operation (cmdl/local-operation cmdl 'CHILD-PORT)))
+  (or (let ((operation (cmdl/local-operation cmdl 'child-port)))
 	(and operation
 	     (operation cmdl)))
       (cmdl/port cmdl)))
@@ -106,55 +111,58 @@ USA.
 	cmdl)))
 
 (define (cmdl/set-default-directory cmdl pathname)
-  (let ((operation (cmdl/local-operation cmdl 'SET-DEFAULT-DIRECTORY)))
+  (let ((operation (cmdl/local-operation cmdl 'set-default-directory)))
     (if operation
 	(operation cmdl pathname)))
   (port/set-default-directory (cmdl/port cmdl) pathname))
 
 (define (cmdl/start cmdl message)
-  (let ((port (cmdl/port cmdl)))
+  (let ((port (cmdl/port cmdl))
+	(pathname-defaults (param:default-pathname-defaults)))
     (let ((thunk
 	   (lambda ()
-	     (fluid-let ((*nearest-cmdl* cmdl)
-			 (dynamic-handler-frames '())
-			 (*bound-restarts*
-			  (if (cmdl/parent cmdl) *bound-restarts* '()))
-			 (standard-error-hook #f)
-			 (standard-warning-hook #f)
-			 (standard-breakpoint-hook #f)
-			 (*working-directory-pathname*
-			  *working-directory-pathname*)
-			 (*default-pathname-defaults*
-			  *default-pathname-defaults*)
-			 (*current-input-port* #f)
-			 (*current-output-port* #f)
-			 (*notification-output-port* #f)
-			 (*trace-output-port* #f)
-			 (*interaction-i/o-port* #f))
-	       (let loop ((message message))
-		 (loop
-		  (bind-abort-restart cmdl
-		    (lambda ()
-		      (deregister-all-events)
-		      (with-interrupt-mask interrupt-mask/all
-			(lambda (interrupt-mask)
-			  interrupt-mask
-			  (unblock-thread-events)
-			  (ignore-errors
-			   (lambda ()
-			     ((->cmdl-message message) cmdl)))
-			  (call-with-current-continuation
-			   (lambda (continuation)
-			     (with-create-thread-continuation continuation
-			       (lambda ()
-				 ((cmdl/driver cmdl) cmdl))))))))))))))
-	  (mutex (port/thread-mutex port)))
+	     (parameterize ((current-input-port #f)
+			    (current-output-port #f)
+			    (notification-output-port #f)
+			    (trace-output-port #f)
+			    (interaction-i/o-port #f)
+			    (working-directory-pathname
+			     (working-directory-pathname))
+			    (current-library-db (current-library-db))
+			    (param:nearest-cmdl cmdl)
+			    (param:standard-error-hook #f)
+			    (param:standard-warning-hook #f)
+			    (param:standard-breakpoint-hook #f)
+			    (param:default-pathname-defaults pathname-defaults)
+			    (dynamic-handler-frames '())
+			    (param:bound-restarts
+			     (if (cmdl/parent cmdl)
+				 (param:bound-restarts)
+				 '())))
+	       (fluid-let ((*default-pathname-defaults* pathname-defaults))
+		 (let loop ((message message))
+		   (loop
+		    (bind-abort-restart cmdl
+		      (lambda ()
+			(with-interrupt-mask interrupt-mask/all
+			  (lambda (interrupt-mask)
+			    interrupt-mask
+			    (unblock-thread-events)
+			    (ignore-errors
+			     (lambda ()
+			       ((->cmdl-message message) cmdl)))
+			    (call-with-current-continuation
+			     (lambda (continuation)
+			       (with-create-thread-continuation continuation
+				 (lambda ()
+				   ((cmdl/driver cmdl) cmdl)))))))))))))))
+	  (mutex (textual-port-thread-mutex port)))
       (let ((thread (current-thread))
 	    (owner (thread-mutex-owner mutex)))
 	(cond ((and owner (not (eq? thread owner)))
 	       (signal-thread-event owner
 		 (let ((signaller
-			(or (cmdl/local-operation cmdl 'START-NON-OWNED)
+			(or (cmdl/local-operation cmdl 'start-non-owned)
 			    (lambda (cmdl thread)
 			      cmdl
 			      (error "Non-owner thread can't start CMDL:"
@@ -165,7 +173,7 @@ USA.
 	       (stop-current-thread))
 	      ((let ((parent (cmdl/parent cmdl)))
 		 (and parent
-		      (cmdl/local-operation parent 'START-CHILD)))
+		      (cmdl/local-operation parent 'start-child)))
 	       => (lambda (operation) (operation cmdl thunk)))
 	      (else
 	       (with-thread-mutex-locked mutex thunk)))))))
@@ -173,7 +181,7 @@ USA.
 (define (bind-abort-restart cmdl thunk)
   (call-with-current-continuation
    (lambda (continuation)
-     (with-restart 'ABORT
+     (with-restart 'abort
 	 (string-append "Return to "
 			(if (repl? cmdl)
 			    "read-eval-print"
@@ -198,26 +206,27 @@ USA.
 (define (cmdl-abort-restart? restart)
   (restart/get restart cmdl-abort-restart-tag))
 
-(define *nearest-cmdl*)
+(define param:nearest-cmdl)
 
 (define (nearest-cmdl)
-  (if (not *nearest-cmdl*) (error "NEAREST-CMDL: no cmdl"))
-  *nearest-cmdl*)
+  (let ((cmdl (param:nearest-cmdl)))
+    (if (not cmdl) (error "NEAREST-CMDL: no cmdl"))
+    cmdl))
 
 (define (nearest-cmdl/port)
-  (let ((cmdl *nearest-cmdl*))
+  (let ((cmdl (param:nearest-cmdl)))
     (if cmdl
 	(cmdl/port cmdl)
-	console-i/o-port)))
+	(console-i/o-port))))
 
 (define (nearest-cmdl/level)
-  (let ((cmdl *nearest-cmdl*))
+  (let ((cmdl (param:nearest-cmdl)))
     (if cmdl
 	(cmdl/level cmdl)
 	0)))
 
 (define (nearest-cmdl/batch-mode?)
-  (let ((cmdl *nearest-cmdl*))
+  (let ((cmdl (param:nearest-cmdl)))
     (if cmdl
 	(cmdl/batch-mode? cmdl)
 	#f)))
@@ -278,7 +287,7 @@ USA.
 
 (define ((cmdl-message/strings . strings) cmdl)
   (let ((port (cmdl/port cmdl)))
-    (port/with-output-terminal-mode port 'COOKED
+    (with-output-port-terminal-mode port 'cooked
       (lambda ()
 	(for-each (lambda (string)
 		    (fresh-line port)
@@ -288,7 +297,7 @@ USA.
 
 (define ((cmdl-message/active actor) cmdl)
   (let ((port (cmdl/port cmdl)))
-    (port/with-output-terminal-mode port 'COOKED
+    (with-output-port-terminal-mode port 'cooked
       (lambda ()
 	(actor port)))))
 
@@ -314,42 +323,42 @@ USA.
 ;;;; Interrupts
 
 (define (cmdl-interrupt/breakpoint)
-  ((or (cmdl/operation (nearest-cmdl) 'INTERRUPT/BREAKPOINT)
+  ((or (cmdl/operation (nearest-cmdl) 'interrupt/breakpoint)
        breakpoint)))
 
 (define (cmdl-interrupt/abort-nearest)
-  ((or (cmdl/operation (nearest-cmdl) 'INTERRUPT/ABORT-NEAREST)
+  ((or (cmdl/operation (nearest-cmdl) 'interrupt/abort-nearest)
        abort->nearest)))
 
 (define (cmdl-interrupt/abort-previous)
-  ((or (cmdl/operation (nearest-cmdl) 'INTERRUPT/ABORT-PREVIOUS)
+  ((or (cmdl/operation (nearest-cmdl) 'interrupt/abort-previous)
        abort->previous)))
 
 (define (cmdl-interrupt/abort-top-level)
-  ((or (cmdl/operation (nearest-cmdl) 'INTERRUPT/ABORT-TOP-LEVEL)
+  ((or (cmdl/operation (nearest-cmdl) 'interrupt/abort-top-level)
        abort->top-level)))
 
 (define (abort->nearest #!optional message)
-  (invoke-abort (let ((restart (find-restart 'ABORT)))
+  (invoke-abort (let ((restart (find-restart 'abort)))
 		  (if (not restart)
-		      (error:no-such-restart 'ABORT))
+		      (error:no-such-restart 'abort))
 		  restart)
 		(if (default-object? message) "Abort!" message)))
 
 (define (abort->previous #!optional message)
-  (invoke-abort (let ((restarts (find-restarts 'ABORT (bound-restarts))))
-		  (let ((next (find-restarts 'ABORT (cdr restarts))))
+  (invoke-abort (let ((restarts (find-restarts 'abort (bound-restarts))))
+		  (let ((next (find-restarts 'abort (cdr restarts))))
 		    (cond ((pair? next) (car next))
 			  ((pair? restarts) (car restarts))
-			  (else (error:no-such-restart 'ABORT)))))
+			  (else (error:no-such-restart 'abort)))))
 		(if (default-object? message) "Up!" message)))
 
 (define (abort->top-level #!optional message)
-  (invoke-abort (let loop ((restarts (find-restarts 'ABORT (bound-restarts))))
-		  (let ((next (find-restarts 'ABORT (cdr restarts))))
+  (invoke-abort (let loop ((restarts (find-restarts 'abort (bound-restarts))))
+		  (let ((next (find-restarts 'abort (cdr restarts))))
 		    (cond ((pair? next) (loop next))
 			  ((pair? restarts) (car restarts))
-			  (else (error:no-such-restart 'ABORT)))))
+			  (else (error:no-such-restart 'abort)))))
 		(if (default-object? message) "Quit!" message)))
 
 (define (find-restarts name restarts)
@@ -366,7 +375,7 @@ USA.
 	(effector))))
 
 (define cmdl-abort-restart-tag
-  (list 'CMDL-ABORT-RESTART-TAG))
+  (list 'cmdl-abort-restart-tag))
 
 ;;;; REP Loops
 
@@ -378,17 +387,17 @@ USA.
 	     (let ((inherit
 		    (let ((repl (and parent (skip-non-repls parent))))
 		      (lambda (argument default name check-arg)
-			(if (eq? 'INHERIT argument)
+			(if (eq? 'inherit argument)
 			    (begin
 			      (if (not repl)
 				  (error "Can't inherit -- no REPL ancestor:"
 					 name))
 			      (default repl))
-			    (check-arg argument 'MAKE-REPL))))))
+			    (check-arg argument 'make-repl))))))
 	       (make-repl-state
-		(inherit (if (default-object? prompt) 'INHERIT prompt)
+		(inherit (if (default-object? prompt) 'inherit prompt)
 			 repl/prompt
-			 'PROMPT
+			 'prompt
 			 (lambda (object procedure)
 			   (if (not (string? object))
 			       (error:wrong-type-argument object
@@ -397,15 +406,15 @@ USA.
 			   object))
 		(inherit environment
 			 repl/environment
-			 'ENVIRONMENT
+			 'environment
 			 ->environment)
 		(if (default-object? condition) #f condition)))
 	     (append (if (default-object? operations) '() operations)
 		     default-repl-operations)))
 
 (define default-repl-operations
-  `((START-CHILD ,(lambda (cmdl thunk) cmdl (with-history-disabled thunk)))
-    (START-NON-OWNED
+  `((start-child ,(lambda (cmdl thunk) cmdl (with-history-disabled thunk)))
+    (start-non-owned
      ,(lambda (repl thread)
 	(let ((condition (repl/condition repl)))
 	  (if condition
@@ -420,12 +429,12 @@ USA.
 	       environment
 	       (if (default-object? condition) #f condition)
 	       (if (default-object? operations) '() operations)
-	       (if (default-object? prompt) 'INHERIT prompt))))
+	       (if (default-object? prompt) 'inherit prompt))))
 
 (define (repl-driver repl)
   (let ((condition (repl/condition repl)))
     (if (and condition (condition/error? condition))
-	(cond ((cmdl/operation repl 'ERROR-DECISION)
+	(cond ((cmdl/operation repl 'error-decision)
 	       => (lambda (operation)
 		    (operation repl condition)))
 	      (hook/error-decision
@@ -435,45 +444,44 @@ USA.
     (do () (#f)
       (if (queue-empty? queue)
 	  (let ((environment (repl/environment repl)))
-	    (%repl-eval/write (hook/repl-read environment repl)
+	    (%repl-eval/write (hook/repl-read repl)
 			      environment
 			      repl))
 	  ((dequeue! queue) repl)))))
 
 (define (run-in-nearest-repl procedure)
-  (guarantee-procedure-of-arity procedure 1 'run-in-nearest-repl)
+  (guarantee unary-procedure? procedure 'run-in-nearest-repl)
   (enqueue! (repl/input-queue (nearest-repl)) procedure))
 
-(define (repl-read #!optional environment repl)
-  (receive (environment repl) (optional-er environment repl 'REPL-READ)
-    (hook/repl-read environment repl)))
+(define (repl-read #!optional repl)
+  (hook/repl-read (optional-repl repl 'repl-read)))
 
 (define hook/repl-read)
-(define (default/repl-read environment repl)
-  (prompt-for-command-expression (cons 'STANDARD (repl/prompt repl))
-				 (cmdl/port repl)
-				 environment))
+(define (default/repl-read repl)
+  (prompt-for-command-expression (cons 'standard (repl/prompt repl))
+				 (cmdl/port repl)))
 
 (define (repl-eval s-expression #!optional environment repl)
-  (receive (environment repl) (optional-er environment repl 'REPL-EVAL)
+  (receive (environment repl) (optional-er environment repl 'repl-eval)
     (%repl-eval s-expression environment repl)))
 
 (define (%repl-eval s-expression environment repl)
   (repl-history/record! (repl/reader-history repl) s-expression)
-  (let ((value (hook/repl-eval s-expression environment repl)))
-    (repl-history/record! (repl/printer-history repl) value)
-    value))
+  (receive vals (hook/repl-eval s-expression environment repl)
+    (for-each (let ((history (repl/printer-history repl)))
+		(lambda (val)
+		  (repl-history/record! history val)))
+	      vals)
+    (apply values vals)))
 
 (define hook/repl-eval)
 (define (default/repl-eval s-expression environment repl)
-  (if (and (pair? s-expression)
-	   (eq? 'UNQUOTE (car s-expression)))
-      (let ((env (->environment '(user))))
-	(%repl-scode-eval (syntax (cadr s-expression) env) env repl))
+  (if (rep-escape? s-expression)
+      (handle-rep-escape s-expression environment)
       (%repl-scode-eval (syntax s-expression environment) environment repl)))
 
 (define (repl-scode-eval scode #!optional environment repl)
-  (receive (environment repl) (optional-er environment repl 'REPL-SCODE-EVAL)
+  (receive (environment repl) (optional-er environment repl 'repl-scode-eval)
     (%repl-scode-eval scode environment repl)))
 
 (define (%repl-scode-eval scode environment repl)
@@ -487,47 +495,42 @@ USA.
    with-repl-eval-boundary
    repl))
 
-(define (repl-write value s-expression #!optional environment repl)
-  (receive (environment repl) (optional-er environment repl 'REPL-WRITE)
-    (hook/repl-write value s-expression environment repl)))
+(define (repl-write vals s-expression #!optional repl)
+  (hook/repl-write vals s-expression (optional-repl repl 'repl-write)))
 
 (define hook/repl-write)
-(define (default/repl-write object s-expression environment repl)
-  (port/write-result (cmdl/port repl)
-		     s-expression
-		     object
-		     (and repl:write-result-hash-numbers?
-			  (object-pointer? object)
-			  (not (interned-symbol? object))
-			  (not (number? object))
-			  (object-hash object))
-		     environment))
+(define (default/repl-write vals s-expression repl)
+  (port/write-values (cmdl/port repl) s-expression vals))
 
 (define (repl-eval/write s-expression #!optional environment repl)
-  (receive (environment repl) (optional-er environment repl 'REPL-EVAL/WRITE)
+  (receive (environment repl) (optional-er environment repl 'repl-eval/write)
     (%repl-eval/write s-expression environment repl)))
 
 (define (%repl-eval/write s-expression environment repl)
-  (hook/repl-write (%repl-eval s-expression environment repl)
-		   s-expression
-		   environment
-		   repl))
+  (receive vals (%repl-eval s-expression environment repl)
+    (hook/repl-write vals s-expression repl)))
+
+(define (optional-repl repl caller)
+  (if (default-object? repl)
+      (nearest-repl)
+      (guarantee repl? repl caller)))
 
 (define (optional-er environment repl caller)
-  (let ((repl
-	 (if (default-object? repl)
-	     (nearest-repl)
-	     (begin
-	       (guarantee-repl repl caller)
-	       repl))))
+  (let ((repl (optional-repl repl caller)))
     (values (if (default-object? environment)
 		(repl/environment repl)
-		(begin
-		  (guarantee-environment environment caller)
-		  environment))
+		(guarantee environment? environment caller))
 	    repl)))
+
+(define (repl-get-hash-number object)
+  (and repl:write-result-hash-numbers?
+       (object-pointer? object)
+       (not (interned-symbol? object))
+       (not (number? object))
+       (hash-object object)))
 
 (define (repl/start repl #!optional message)
+  ((repl/env-mgr repl) 'set-repl! repl)
   (cmdl/start repl
 	      (make-repl-message repl
 				 (if (default-object? message)
@@ -540,9 +543,9 @@ USA.
      (or message
 	 (and condition
 	      (cmdl-message/strings
-	       (fluid-let ((*unparser-list-depth-limit* 25)
-			   (*unparser-list-breadth-limit* 100)
-			   (*unparser-string-length-limit* 500))
+	       (parameterize ((param:printer-list-depth-limit 25)
+			      (param:printer-list-breadth-limit 100)
+			      (param:printer-string-length-limit 500))
 		 (condition/report-string condition)))))
      (and condition
 	  repl:allow-restart-notifications?
@@ -557,15 +560,15 @@ USA.
     (if (not (and parent
 		  (repl? parent)
 		  (eq? (repl/environment parent) environment)))
-	(let ((operation (cmdl/operation repl 'SET-DEFAULT-ENVIRONMENT)))
+	(let ((operation (cmdl/operation repl 'set-default-environment)))
 	  (if operation
 	      (operation repl environment)
 	      (hook/set-default-environment repl environment))))))
 
 (define hook/set-default-environment)
-(define (default/set-default-environment port environment)
-  (let ((port (cmdl/port port)))
-    (port/with-output-terminal-mode port 'COOKED
+(define (default/set-default-environment repl environment)
+  (let ((port (cmdl/port repl)))
+    (with-output-port-terminal-mode port 'cooked
       (lambda ()
 	(if (not (interpreter-environment? environment))
 	    (begin
@@ -625,9 +628,9 @@ USA.
 			     (loop))))))
 		 (begin
 		   (if (not (exact-integer? n))
-		       (error:wrong-type-argument n "exact integer" 'RESTART))
+		       (error:wrong-type-argument n "exact integer" 'restart))
 		   (if (not (<= 1 n n-restarts))
-		       (error:bad-range-argument n 'RESTART))
+		       (error:bad-range-argument n 'restart))
 		   n))))
 	 condition)))))
 
@@ -646,7 +649,7 @@ USA.
     (if (pair? restarts)
 	(let ((rest
 	       (if (cmdl-abort-restart? (car restarts))
-		   (list-transform-positive (cdr restarts) cmdl-abort-restart?)
+		   (filter cmdl-abort-restart? (cdr restarts))
 		   (loop (cdr restarts)))))
 	  (if (restart/interactor (car restarts))
 	      (cons (car restarts) rest)
@@ -664,53 +667,48 @@ USA.
 	 (write index port)
 	 (write-string ") =>" port))))))
 
-(define-structure (repl-state
-		   (conc-name repl-state/)
-		   (constructor make-repl-state
-				(prompt environment condition)))
-  prompt
-  environment
-  (condition #f read-only #t)
-  (reader-history (make-repl-history repl-reader-history-size))
-  (printer-history (make-repl-history repl-printer-history-size))
-  (input-queue (make-queue) read-only #t))
+(define (make-repl-state prompt environment condition)
+  (%make-repl-state prompt
+		    (make-env-mgr environment)
+		    condition
+		    (make-repl-history repl-reader-history-size)
+		    (make-repl-history repl-reader-history-size)
+		    (make-serial-queue)))
+
+(define-record-type <repl-state>
+    (%make-repl-state prompt env-mgr condition reader-history printer-history
+		      input-queue)
+    repl-state?
+  (prompt repl-state/prompt set-repl-state/prompt!)
+  (env-mgr repl-state/env-mgr)
+  (condition repl-state/condition)
+  (reader-history repl-state/reader-history)
+  (printer-history repl-state/printer-history)
+  (input-queue repl-state/input-queue))
 
 (define (repl? object)
   (and (cmdl? object)
        (repl-state? (cmdl/state object))))
 
-(define-guarantee repl "read-eval-print loop")
-
-(define-integrable (repl/prompt repl)
+(define (repl/prompt repl)
   (repl-state/prompt (cmdl/state repl)))
 
-(define-integrable (set-repl/prompt! repl prompt)
+(define (set-repl/prompt! repl prompt)
   (set-repl-state/prompt! (cmdl/state repl) prompt))
 
-(define-integrable (repl/environment repl)
-  (repl-state/environment (cmdl/state repl)))
+(define (repl/env-mgr repl)
+  (repl-state/env-mgr (cmdl/state repl)))
 
-(define (set-repl/environment! repl environment)
-  (set-repl-state/environment! (cmdl/state repl) environment)
-  (repl/set-default-environment repl)
-  (port/set-default-environment (cmdl/port repl) environment))
-
-(define-integrable (repl/condition repl)
+(define (repl/condition repl)
   (repl-state/condition (cmdl/state repl)))
 
-(define-integrable (repl/reader-history repl)
+(define (repl/reader-history repl)
   (repl-state/reader-history (cmdl/state repl)))
 
-(define-integrable (set-repl/reader-history! repl reader-history)
-  (set-repl-state/reader-history! (cmdl/state repl) reader-history))
-
-(define-integrable (repl/printer-history repl)
+(define (repl/printer-history repl)
   (repl-state/printer-history (cmdl/state repl)))
 
-(define-integrable (set-repl/printer-history! repl printer-history)
-  (set-repl-state/printer-history! (cmdl/state repl) printer-history))
-
-(define-integrable (repl/input-queue repl)
+(define (repl/input-queue repl)
   (repl-state/input-queue (cmdl/state repl)))
 
 (define (repl/parent repl)
@@ -726,17 +724,114 @@ USA.
 	   cmdl
 	   (skip-non-repls (cmdl/parent cmdl)))))
 
-(define (repl/base repl)
-  (let ((parent (repl/parent repl)))
-    (if parent
-	(repl/base parent)
-	repl)))
+(define (nearest-repl/condition)
+  (repl/condition (nearest-repl)))
 
 (define (nearest-repl/environment)
   (repl/environment (nearest-repl)))
 
-(define (nearest-repl/condition)
-  (repl/condition (nearest-repl)))
+(define (repl/environment repl)
+  ((repl/env-mgr repl) 'current))
+
+(define (set-repl/environment! repl environment)
+  ((repl/env-mgr repl) 'push! environment))
+
+(define (nearest-repl/named-environment name)
+  ((repl/env-mgr (nearest-repl)) 'get-named name))
+
+(define (make-env-mgr environment)
+  (let ((repl #f)
+	(current-env environment)
+	(stack '())
+	(table (alist-table eq?)))
+
+    (define (set-repl! repl*)
+      (set! repl repl*)
+      unspecific)
+
+    (define (current)
+      current-env)
+
+    (define (set-current! env)
+      (set! current-env env)
+      (repl/set-default-environment repl)
+      (port/set-default-environment (cmdl/port repl) env)
+      env)
+
+    (define (get-stack)
+      (weak-list->list stack))
+
+    (define (push! env)
+      (guarantee environment? env)
+      (if (eq? current-env env)
+	  env
+	  (begin
+	    (set! stack (weak-cons current-env (weak-delq! env stack)))
+	    (set-current! env))))
+
+    (define (pop!)
+      (%pop-stack! set-current!))
+
+    (define (swap-tos!)
+      (%pop-stack! (lambda (env)
+		     (set! stack (weak-cons current-env stack))
+		     (set-current! env))))
+
+    (define (bury!)
+      (%pop-stack! (lambda (env)
+		     (let ((tail (weak-cons current-env '()))
+			   (p (weak-last-pair stack)))
+		       (if p
+			   (weak-set-cdr! p tail)
+			   (set! stack tail)))
+		     (set-current! env))))
+
+    (define (%pop-stack! succeed)
+      (let loop ((this stack))
+	(if (weak-pair? this)
+	    (let ((env (weak-car this))
+		  (next (weak-cdr this)))
+	      (set! stack next)
+	      (if (gc-reclaimed-object? env)
+		  (loop next)
+		  (begin (succeed env) #t)))
+	    #f)))
+
+    (define (known-name? name)
+      (alist-table-contains? table (guarantee interned-symbol? name)))
+
+    (define (get-named name)
+      (alist-table-ref table (guarantee interned-symbol? name)))
+
+    (define (name-current! name)
+      (alist-table-set! table (guarantee interned-symbol? name) current-env))
+
+    (define (delete-name! name)
+      (alist-table-delete! table (guarantee interned-symbol? name)))
+
+    (define (clear-names!)
+      (alist-table-clear! table))
+
+    (define (env-names)
+      (alist-table-keys table))
+
+    (define (named-envs)
+      (alist-table->alist table))
+
+    (define (name-of env)
+      (guarantee environment? env)
+      (alist-table-search table
+			  (lambda (name env*)
+			    (declare (ignore name))
+			    (eq? env env*))
+			  (lambda (name env*)
+			    (declare (ignore env*))
+			    name)
+			  (lambda () #f)))
+
+    (bundle #f set-repl! current set-current! get-stack push! pop! swap-tos!
+	    bury! known-name? get-named name-current! delete-name! clear-names!
+	    env-names named-envs name-of)))
 
 ;;;; History
 
@@ -761,48 +856,33 @@ USA.
 (define (repl-history/replace-current! history object)
   (let ((elements (repl-history/elements history)))
     (if (pair? elements)
-	(set-car! (list-tail elements (- (repl-history/size history) 1))
+	(set-car! (drop elements (- (repl-history/size history) 1))
 		  object))))
 
 (define (repl-history/read history n)
   (if (not (and (exact-nonnegative-integer? n)
 		(< n (repl-history/size history))))
-      (error:wrong-type-argument n "history index" 'REPL-HISTORY/READ))
+      (error:wrong-type-argument n "history index" 'repl-history/read))
   (list-ref (repl-history/elements history)
 	    (- (- (repl-history/size history) 1) n)))
 
-;;; User Interface Stuff
+;;; User interface
 
 (define (pe)
-  (let ((environment (nearest-repl/environment)))
-    (let ((package (environment->package environment)))
-      (if package
-	  (package/name package)
-	  environment))))
+  (let ((env-mgr (repl/env-mgr (nearest-repl))))
+    (let ((env (env-mgr 'current)))
+      (or (env-mgr 'name-of env)
+	  (let ((package (environment->package env)))
+	    (if package
+		(package/name package)
+		env))))))
 
-(define (ge environment)
-  (let ((environment (->environment environment 'GE)))
-    (set-repl/environment! (nearest-repl) environment)
-    (set-load-environment! environment)
-    environment))
-
-(define (->environment object #!optional caller)
-  (let ((caller (if (default-object? caller) '->ENVIRONMENT caller)))
-    (cond ((environment? object) object)
-	  ((package? object) (package/environment object))
-	  ((procedure? object) (procedure-environment object))
-	  ((promise? object) (promise-environment object))
-	  (else
-	   (let ((package
-		  (let ((package-name
-			 (cond ((symbol? object) (list object))
-			       ((list? object) object)
-			       (else #f))))
-		    (and package-name
-			 (name->package package-name)))))
-	     (if (not package)
-		 (error:wrong-type-argument object "environment" caller))
-	     (package/environment package))))))
+(define (ge #!optional environment)
+  ((repl/env-mgr (nearest-repl))
+   'set-current!
+   (if (default-object? environment)
+       (make-top-level-environment)
+       (->environment environment 'ge))))
 
 (define (re #!optional index)
   (repl-eval (repl-history/read (repl/reader-history (nearest-repl))
@@ -819,8 +899,12 @@ USA.
 (define (read-eval-print environment message prompt)
   (repl/start (push-repl environment #f '() prompt) message))
 
-(define (ve environment)
-  (read-eval-print (->environment environment 'VE) #f 'INHERIT))
+(define (ve #!optional environment)
+  (read-eval-print (if (default-object? environment)
+		       (make-top-level-environment)
+		       (->environment environment 've))
+		   #f
+		   'inherit))
 
 (define (proceed #!optional value)
   (if (default-object? value)
@@ -833,7 +917,7 @@ USA.
 ;;;; Breakpoints
 
 (define (bkpt datum . arguments)
-  (apply breakpoint-procedure 'CONTINUATION-ENVIRONMENT datum arguments))
+  (apply breakpoint-procedure 'continuation-environment datum arguments))
 
 (define (breakpoint-procedure environment datum . arguments)
   (signal-breakpoint-1 #f
@@ -867,7 +951,7 @@ USA.
   (call-with-current-continuation
    (lambda (restart-continuation)
      (let ((continuation (or continuation restart-continuation)))
-       (with-restart 'CONTINUE reporter
+       (with-restart 'continue reporter
 	   (lambda () (restart-continuation unspecific))
 	   values
 	 (lambda ()
@@ -882,10 +966,10 @@ USA.
 
 (define (get-breakpoint-environment continuation environment message)
   (let ((environment
-	 (if (eq? 'CONTINUATION-ENVIRONMENT environment)
+	 (if (eq? 'continuation-environment environment)
 	     (continuation/first-subproblem-environment continuation)
 	     environment)))
-    (if (eq? 'NO-ENVIRONMENT environment)
+    (if (eq? 'no-environment environment)
 	(values (nearest-repl/environment)
 		(cmdl-message/append
 		 message
@@ -901,9 +985,9 @@ USA.
 	  (lambda (expression environment subexpression)
 	    expression subexpression
 	    (if (debugging-info/undefined-environment? environment)
-		'NO-ENVIRONMENT
+		'no-environment
 		environment)))
-	'NO-ENVIRONMENT)))
+	'no-environment)))
 
 (define condition-type:breakpoint)
 (define condition/breakpoint?)
@@ -911,45 +995,50 @@ USA.
 (define breakpoint/message)
 (define breakpoint/prompt)
 (define %signal-breakpoint)
+(define param:standard-breakpoint-hook)
+(define standard-breakpoint-hook #!default)
 
 (define (initialize-breakpoint-condition!)
   (set! condition-type:breakpoint
-	(make-condition-type 'BREAKPOINT #f '(ENVIRONMENT MESSAGE PROMPT)
+	(make-condition-type 'breakpoint #f '(environment message prompt)
 	  (lambda (condition port)
 	    condition
 	    (write-string "Breakpoint." port))))
   (set! condition/breakpoint?
 	(condition-predicate condition-type:breakpoint))
   (set! breakpoint/environment
-	(condition-accessor condition-type:breakpoint 'ENVIRONMENT))
+	(condition-accessor condition-type:breakpoint 'environment))
   (set! breakpoint/message
-	(condition-accessor condition-type:breakpoint 'MESSAGE))
+	(condition-accessor condition-type:breakpoint 'message))
   (set! breakpoint/prompt
-	(condition-accessor condition-type:breakpoint 'PROMPT))
+	(condition-accessor condition-type:breakpoint 'prompt))
   (set! %signal-breakpoint
 	(let ((make-condition
 	       (condition-constructor condition-type:breakpoint
-				      '(ENVIRONMENT MESSAGE PROMPT))))
+				      '(environment message prompt))))
 	  (lambda (continuation environment message prompt)
 	    (let ((condition
 		   (make-condition continuation
-				   'BOUND-RESTARTS
+				   'bound-restarts
 				   environment
 				   message
 				   prompt)))
 	      (signal-condition condition)
 	      (standard-breakpoint-handler condition)))))
+  (set! param:standard-breakpoint-hook (make-settable-parameter #f))
   unspecific)
 
 (define (standard-breakpoint-handler condition)
-  (let ((hook standard-breakpoint-hook))
+  (let ((hook
+	 (if (default-object? standard-breakpoint-hook)
+	     (param:standard-breakpoint-hook)
+	     standard-breakpoint-hook)))
     (if hook
-	(fluid-let ((standard-breakpoint-hook #f))
-	  (hook condition))))
+	(fluid-let ((standard-breakpoint-hook #!default))
+	  (parameterize ((param:standard-breakpoint-hook #f))
+	    (hook condition)))))
   (repl/start (push-repl (breakpoint/environment condition)
 			 condition
 			 '()
 			 (breakpoint/prompt condition))
-	      (breakpoint/message condition)))
-
-(define standard-breakpoint-hook #f)
\ No newline at end of file
+	      (breakpoint/message condition)))
\ No newline at end of file
