@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -40,6 +40,18 @@ USA.
 (define (transform/recursive block top-level-block expression)
   (transform/top-level-1 false top-level-block block expression))
 
+(define (transform/r7rs-library imports expression)
+  (let ((block (block/make #f #f '())))
+    (for-each (lambda (import)
+		(variable/make&bind! block (library-ixport-to import)))
+	      imports)
+    (set-block/declarations! block (r7rs-usual-integrations block imports))
+    (values block
+	    (transform/top-level-1 'r7rs
+				   block
+				   (block/make block #f '())
+				   expression))))
+
 (define top-level?)
 (define top-level-block)
 (define root-block)
@@ -57,19 +69,18 @@ USA.
 	  (begin
 	    (if (not top-level?)
 		(error "Open blocks allowed only at top level:" expression))
-	    (call-with-values
-		(lambda () (open-block-components expression values))
-	      (lambda (auxiliary declarations body)
-		(if (not (assq 'USUAL-INTEGRATIONS declarations))
-		    (ui-warning))
-		(transform/open-block* expression
-				       block
-				       environment
-				       auxiliary
-				       declarations
-				       body))))
+	    (let ((declarations (scode-open-block-declarations expression)))
+	      (if (not (or (eq? tl? 'r7rs)
+			   (assq 'usual-integrations declarations)))
+		  (ui-warning))
+	      (transform/open-block* expression
+				     block
+				     environment
+				     (scode-open-block-names expression)
+				     declarations
+				     (scode-open-block-actions expression))))
 	  (transform/expression block environment expression)))))
-
+
 (define (ui-warning)
   (for-each
    (lambda (line)
@@ -103,20 +114,19 @@ USA.
 	    (variable/make&bind! top-level-block name)))))
 
 (define (environment/bind environment variables)
-  (map* environment
-	(lambda (variable)
-	  (cons (variable/name variable) variable))
-	variables))
+  (fold-right (lambda (variable env)
+		(cons (cons (variable/name variable) variable)
+		      env))
+	      environment
+	      variables))
 
 (define (transform/open-block block environment expression)
-  (call-with-values (lambda () (open-block-components expression values))
-    (lambda (auxiliary declarations body)
-      (transform/open-block* expression
-			     (block/make block true '())
-			     environment
-			     auxiliary
-			     declarations
-			     body))))
+  (transform/open-block* expression
+			 (block/make block true '())
+			 environment
+			 (scode-open-block-names expression)
+			 (scode-open-block-declarations expression)
+			 (scode-open-block-actions expression)))
 
 (define (transform/open-block* expression block environment auxiliary
 			       declarations body)
@@ -132,7 +142,7 @@ USA.
 		     (transform/expression block environment subexpression))))
 	      (let loop
 		  ((variables variables)
-		   (actions (sequence-actions body)))
+		   (actions (scode-sequence-actions body)))
 		(cond ((null? variables)
 		       (values '() (map transform actions)))
 		      ((null? actions)
@@ -142,13 +152,14 @@ USA.
 		      ;; encounter them in that same order when
 		      ;; looking through the body's actions.
 		      ((and (scode-assignment? (car actions))
-			    (eq? (assignment-name (car actions))
+			    (eq? (scode-assignment-name (car actions))
 				 (variable/name (car variables))))
 		       (call-with-values
 			   (lambda () (loop (cdr variables) (cdr actions)))
 			 (lambda (vals actions*)
 			   (values
-			    (cons (transform (assignment-value (car actions)))
+			    (cons (transform
+				   (scode-assignment-value (car actions)))
 				  vals)
 			    (cons open-block/value-marker actions*)))))
 		      (else
@@ -165,17 +176,18 @@ USA.
   (reference/make expression
 		  block
 		  (environment/lookup environment
-				      (variable-name expression))))
+				      (scode-variable-name expression))
+		  (scode-variable-safe? expression)))
 
 (define (transform/assignment block environment expression)
-  (assignment-components expression
-    (lambda (name value)
-      (let ((variable (environment/lookup environment name)))
-	(variable/side-effect! variable)
-	(assignment/make expression
-			 block
-			 variable
-			 (transform/expression block environment value))))))
+  (let ((name (scode-assignment-name expression))
+	(value (scode-assignment-value expression)))
+    (let ((variable (environment/lookup environment name)))
+      (variable/side-effect! variable)
+      (assignment/make expression
+		       block
+		       variable
+		       (transform/expression block environment value)))))
 
 (define (transform/lambda block environment expression)
   (lambda-components* expression
@@ -193,14 +205,17 @@ USA.
 		   (environment/bind environment
 				     (block/bound-variables block))))
 	      (build-procedure expression block name required optional rest
-			       (transform/procedure-body block environment body)))))))))
+			       (transform/procedure-body block environment
+							 body)))))))))
 
 ;; If procedure body is a sequence, scan the first elements and turn variable
 ;; references into IGNORE declarations.
 (define (build-procedure expression block name required optional rest body)
   (if (sequence? body)
       (do ((actions (sequence/actions body) (cdr actions))
-	   (ignores '() (cons (variable/name (reference/variable (car actions))) ignores)))
+	   (ignores '()
+		    (cons (variable/name (reference/variable (car actions)))
+			  ignores)))
 	  ((or (null? (cdr actions))
 	       (not (reference? (car actions))))
 	   (let ((final-body (if (null? (cdr actions))
@@ -210,93 +225,98 @@ USA.
 	      expression block name required optional rest
 	      (if (null? ignores)
 		  final-body
-		  (declaration/make #f (declarations/parse block `((ignore ,@ignores)))
+		  (declaration/make #f
+				    (declarations/parse block
+							`((ignore ,@ignores)))
 				    final-body))))))
-      (procedure/make
-       expression block name required optional rest
-       body)))
+      (procedure/make expression block name required optional rest body)))
 
 (define (transform/procedure-body block environment expression)
   (if (scode-open-block? expression)
-      (open-block-components expression
-	(lambda (auxiliary declarations body)
-	  (if (null? auxiliary)
-	      (begin (set-block/declarations!
-		      block
-		      (declarations/parse block declarations))
-		     (transform/expression block environment body))
-	      (transform/open-block block environment expression))))
+      (if (null? (scode-open-block-names expression))
+	  (begin
+	    (set-block/declarations!
+	     block
+	     (declarations/parse block
+				 (scode-open-block-declarations expression)))
+	    (transform/expression block
+				  environment
+				  (scode-open-block-actions expression)))
+	  (transform/open-block block environment expression))
       (transform/expression block environment expression)))
 
 (define (transform/definition block environment expression)
-  (definition-components expression
-    (lambda (name value)
-      (if (not (eq? block top-level-block))
-	  (error "Unscanned definition encountered (unable to proceed):" name))
-      (transform/combination*
-       expression block environment
-       (make-combination (make-primitive-procedure 'LOCAL-ASSIGNMENT)
-			 (list (make-the-environment) name value))))))
+  (let ((name (scode-definition-name expression))
+	(value (scode-definition-value expression)))
+    (if (not (eq? block top-level-block))
+	(error "Unscanned definition encountered (unable to proceed):" name))
+    (transform/combination*
+     expression block environment
+     (make-scode-combination
+      (make-primitive-procedure 'local-assignment)
+      (list (make-scode-the-environment) name value)))))
 
 (define (transform/access block environment expression)
-  (access-components expression
-    (lambda (environment* name)
-      (access/make expression
-		   block
-		   (transform/expression block environment environment*)
-		   name))))
+  (access/make expression
+	       block
+	       (transform/expression block
+				     environment
+				     (scode-access-environment expression))
+	       (scode-access-name expression)))
 
 (define (transform/combination block environment expression)
   (transform/combination* expression block environment expression))
 
 (define (transform/combination* expression block environment expression*)
-  (combination-components expression*
-    (lambda (operator operands)
-      (combination/%make expression
-			 block
-			 (transform/expression block environment operator)
-			 (transform/expressions block environment operands)))))
+  (let ((operator (scode-combination-operator expression*))
+	(operands (scode-combination-operands expression*)))
+    (combination/%make expression
+		       block
+		       (transform/expression block environment operator)
+		       (transform/expressions block environment operands))))
 
 (define (transform/comment block environment expression)
-  (transform/expression block environment (comment-expression expression)))
+  (transform/expression block environment
+			(scode-comment-expression expression)))
 
 (define (transform/conditional block environment expression)
-  (conditional-components expression
-    (lambda (predicate consequent alternative)
-      (conditional/make
-       expression
-       (transform/expression block environment predicate)
-       (transform/expression block environment consequent)
-       (transform/expression block environment alternative)))))
+  (let ((predicate (scode-conditional-predicate expression))
+	(consequent (scode-conditional-consequent expression))
+	(alternative (scode-conditional-alternative expression)))
+    (conditional/make
+     expression
+     (transform/expression block environment predicate)
+     (transform/expression block environment consequent)
+     (transform/expression block environment alternative))))
 
 (define (transform/constant block environment expression)
   block environment ; ignored
   (constant/make expression expression))
 
 (define (transform/declaration block environment expression)
-  (declaration-components expression
-    (lambda (declarations expression*)
-      (declaration/make expression
-			(declarations/parse block declarations)
-			(transform/expression block environment
-					      expression*)))))
+  (declaration/make
+   expression
+   (declarations/parse block (scode-declaration-text expression))
+   (transform/expression block environment
+			 (scode-declaration-expression expression))))
 
 (define (transform/delay block environment expression)
   (delay/make
    expression
-   (transform/expression block environment (delay-expression expression))))
+   (transform/expression block environment
+			 (scode-delay-expression expression))))
 
 (define (transform/disjunction block environment expression)
-  (disjunction-components expression
-    (lambda (predicate alternative)
-      (disjunction/make
-       expression
-       (transform/expression block environment predicate)
-       (transform/expression block environment alternative)))))
+  (disjunction/make
+   expression
+   (transform/expression block environment
+			 (scode-disjunction-predicate expression))
+   (transform/expression block environment
+			 (scode-disjunction-alternative expression))))
 
 (define (transform/quotation block environment expression)
   block environment			;ignored
-  (transform/quotation* expression (quotation-expression expression)))
+  (transform/quotation* expression (scode-quotation-expression expression)))
 
 (define (transform/quotation* expression expression*)
   (call-with-values (lambda () (transform/top-level expression* '()))
@@ -306,9 +326,15 @@ USA.
 (define (transform/sequence block environment expression)
   ;; Don't remove references from sequences here.  We want them
   ;; to signal ignored variables.
-  (sequence/%make
-   expression
-   (transform/expressions block environment (sequence-actions expression))))
+  (let ((actions
+	 (transform/expressions block environment
+				(scode-sequence-actions expression))))
+    (cond ((not (pair? actions))
+	   (transform/constant block environment unspecific))
+	  ((not (pair? (cdr actions)))
+	   (car actions))
+	  (else
+	   (sequence/%make expression actions)))))
 
 (define (transform/the-environment block environment expression)
   environment ; ignored
@@ -318,18 +344,18 @@ USA.
 (define transform/dispatch
   (make-scode-walker
    transform/constant
-   `((ACCESS ,transform/access)
-     (ASSIGNMENT ,transform/assignment)
-     (COMBINATION ,transform/combination)
-     (COMMENT ,transform/comment)
-     (CONDITIONAL ,transform/conditional)
-     (DECLARATION ,transform/declaration)
-     (DEFINITION ,transform/definition)
-     (DELAY ,transform/delay)
-     (DISJUNCTION ,transform/disjunction)
-     (LAMBDA ,transform/lambda)
-     (OPEN-BLOCK ,transform/open-block)
-     (QUOTATION ,transform/quotation)
-     (SEQUENCE ,transform/sequence)
-     (THE-ENVIRONMENT ,transform/the-environment)
-     (VARIABLE ,transform/variable))))
\ No newline at end of file
+   `((access ,transform/access)
+     (assignment ,transform/assignment)
+     (combination ,transform/combination)
+     (comment ,transform/comment)
+     (conditional ,transform/conditional)
+     (declaration ,transform/declaration)
+     (definition ,transform/definition)
+     (delay ,transform/delay)
+     (disjunction ,transform/disjunction)
+     (lambda ,transform/lambda)
+     (open-block ,transform/open-block)
+     (quotation ,transform/quotation)
+     (sequence ,transform/sequence)
+     (the-environment ,transform/the-environment)
+     (variable ,transform/variable))))
\ No newline at end of file
