@@ -1,8 +1,10 @@
 #!/bin/sh
 #
-# $Id: Tags.sh,v 1.4 2003/02/14 18:48:11 cph Exp $
-#
-# Copyright 2000 Massachusetts Institute of Technology
+# Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+#     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
+#     2015, 2016, 2017, 2018, 2019, 2020 Massachusetts Institute of
+#     Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -18,10 +20,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with MIT/GNU Scheme; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+# 02110-1301, USA.
 
 # Utility to make TAGS file for an MIT/GNU Scheme build directory.
 # The working directory must be the build directory.
 
-etags *.scm
+rm TAGS
+find * -type f \( -name \*.scm -o -name \*.sld -o -name \*.pkg \) -depth 0 -print0 \
+    | xargs -0 etags --append --language=scheme
